/*  
 *  This file is part of w3panik
 * 
 *  w3panik - (C) 2018 Ariela Wenner - <arisunz@disroot.org>
 *  
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <https://www.gnu.org/licenses/>.
 * 
 */

#include <stdio.h>
#include <fcntl.h>
#include <unistd.h>
#include <linux/input.h>
#include "Keybuffer.h"
#include "Commands.h"
 
//#define KEY_Ñ 39

unsigned char event_check(uint16_t value);

int main(int argc, char* argv[]) {
    if (argc != 2) {
        fprintf(stderr, "Usage: ./w3panik <keyboard event* number>\n");
        fprintf(stderr, "Find out your corresponding event with `ls -l /dev/input/by-id/'\n");
        return EXIT_FAILURE;
    }
    
    if (getuid() != 0) {
        fprintf(stderr, "For simplicity and security's sake, this programm needs to be run as root\n");
        return EXIT_FAILURE;
    }
    
    uint8_t event_path[20] = "/dev/input/event";
    strcat(event_path, argv[1]);
    printf("%s\n", event_path);
    
    FILE* event_stream;
    
    // TODO: Set max sizes from config file
    keydeque* key_deque = buffer_allocate(32);
    
    uint8_t* klist[8];
    uint8_t* clist[8];
    size_t list_size = 0;
    
    command_list_append(klist, clist, &list_size, "EXAMPLE", "notify-send example");
    command_list_append(klist, clist, &list_size, "WOW", "notify-send wow");
    command_list_append(klist, clist, &list_size, "OWO", "notify-send owo");
    
    if (event_stream = fopen(event_path, "rb")) {
        printf("Event stream opened!\n");
    } else {
        printf("Failed to open event stream stream...\n");
    }
    
    struct input_event* ev = (struct input_event*) malloc(sizeof(struct input_event));
    while (fread(ev, sizeof(struct input_event), 1, event_stream)) {
        if (ev->value == 1) {
            if (ev->code == 1) { // escape
                break;
            }
            
            unsigned char ch_code = event_check(ev->code);
            if (ch_code != ' ') {
                buffer_append(key_deque, ch_code);
                //fprintf(stderr, "%s\n", key_deque->buffer);
                if (command_check(klist, clist, list_size, key_deque->buffer) == 0) {
                    buffer_clear(key_deque);
                }
            }
        }
    }

    if (fclose(event_stream) != 0) {
        printf("Failed to close event stream...\n");
    } else {
        printf("Event stream closed!\n");
    }

    free(ev);
    free(key_deque);
    
    return EXIT_SUCCESS;
}

unsigned char event_check(uint16_t value) {
    switch (value) {
        case KEY_1: return '1';
        case KEY_2: return '2';
        case KEY_3: return '3';
        case KEY_4: return '4';
        case KEY_5: return '5';
        case KEY_6: return '6';
        case KEY_7: return '7';
        case KEY_8: return '8';
        case KEY_9: return '9';
        case KEY_0: return '0';
        case KEY_Q: return 'Q';
        case KEY_W: return 'W';
        case KEY_E: return 'E';
        case KEY_R: return 'R';
        case KEY_T: return 'T';
        case KEY_Y: return 'Y';
        case KEY_U: return 'U';
        case KEY_I: return 'I';
        case KEY_O: return 'O';
        case KEY_P: return 'P';
        case KEY_A: return 'A';
        case KEY_S: return 'S';
        case KEY_D: return 'D';
        case KEY_F: return 'F';
        case KEY_G: return 'G';
        case KEY_H: return 'H';
        case KEY_J: return 'J';
        case KEY_K: return 'K';
        case KEY_L: return 'L';
        //case KEY_Ñ: return 'Ñ';
        case KEY_Z: return 'Z';
        case KEY_X: return 'X';
        case KEY_C: return 'C';
        case KEY_V: return 'V';
        case KEY_B: return 'B';
        case KEY_N: return 'N';
        case KEY_M: return 'M';
        default: return ' ';
    }
}
